/*******************************************************************************
*
* E M B E D D E D   W I Z A R D   P R O J E C T
*
*                                                Copyright (c) TARA Systems GmbH
*                                    written by Paul Banach and Manfred Schweyer
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* TARA Systems hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   This file is part of the interface (glue layer) between an Embedded Wizard
*   generated UI application and the board support package (BSP) of a dedicated
*   target. It is intended to be used as a template. Feel free to adapt this
*   implementation according your particular hardware.
*
*   This file is responsible to initialize the display hardware and to provide
*   the necessary access to update the display content. The color format of the
*   framebuffer has to correspond to the color format of the Graphics Engine.
*
*******************************************************************************/

#include <string.h>
#include <windows.h>

#include "ewrte.h"
#include "ewgfx.h"

#include "ew_bsp_display.h"
#include "gfx_system.h"


/*******************************************************************************
* FUNCTION:
*   EwBspDisplayInit
*
* DESCRIPTION:
*   The function EwBspDisplayInit initializes the display hardware and returns
*   the display parameter.
*
* ARGUMENTS:
*   aGuiWidth,
*   aGuiHeight   - Size of the GUI in pixel.
*   aDisplayInfo - Display info data structure.
*
* RETURN VALUE:
*   Returns 1 if successful, 0 otherwise.
*
*******************************************************************************/
int EwBspDisplayInit( int aGuiWidth, int aGuiHeight, XDisplayInfo* aDisplayInfo )
{
  int            guiWidth;
  int            guiHeight;
  XGfxSystemInfo gfxSystemInfo;

  /* check and clean display info structure */
  if ( !aDisplayInfo )
    return 0;

  memset( aDisplayInfo, 0, sizeof( XDisplayInfo ));

  /* adjust size of gui if viewport is rotated */
  #if (( EW_SURFACE_ROTATION == 90 ) || ( EW_SURFACE_ROTATION == 270 ))
    guiWidth  = aGuiHeight;
    guiHeight = aGuiWidth;
  #else
    guiWidth  = aGuiWidth;
    guiHeight = aGuiHeight;
  #endif

  /* create a surface */
  if ( ! GfxSystemCreateSurface( guiWidth, guiHeight ))
  {
    EwPrint( "Cannot create surface\n" );
    return 0;
  }


  /* get gfx system info structure */
  GfxSystemGetInfo( &gfxSystemInfo );

  if ( gfxSystemInfo.Type != GFX_SYSTEM_WINDOWS )
  {
    EwPrint( "Invalid GFX system\n" );
    return 0;
  }

  /* return the current display configuration */
  aDisplayInfo->FrameBuffer    = gfxSystemInfo.FrameBuffer;
  aDisplayInfo->DisplayWidth   = guiWidth;
  aDisplayInfo->DisplayHeight  = guiHeight;
  aDisplayInfo->ViewportX      = 0;
  aDisplayInfo->ViewportY      = 0;
  aDisplayInfo->ViewportWidth  = guiWidth;
  aDisplayInfo->ViewportHeight = guiHeight;
  aDisplayInfo->Context        = 0;

  return 1;
}


/*******************************************************************************
* FUNCTION:
*   EwBspDisplayDone
*
* DESCRIPTION:
*   The function EwBspDisplayDone deinitializes the display hardware.
*
* ARGUMENTS:
*   aDisplayInfo - Display info data structure.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspDisplayDone( XDisplayInfo* aDisplayInfo )
{
  EW_UNUSED_ARG( aDisplayInfo );
}


/*******************************************************************************
* FUNCTION:
*   EwBspDisplayUpdate
*
* DESCRIPTION:
*   The function EwBspDisplayUpdate updates the screen content.
*
* ARGUMENTS:
*   aContext - Display context.
*   aArea    - The update area.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspDisplayUpdate( void* aContext, XRect aArea )
{
  /* start an update if there is anything to draw */
  if ( !EwIsRectEmpty( aArea ))
    GfxSystemUpdateSurface(
      aArea.Point1.X, aArea.Point1.Y, aArea.Point2.X, aArea.Point2.Y );
}


/* mli, msy */
